@@ -1,5 +1,5 @@
 '''Frame is the Client Framework that verifies and installs entries, and generates statistics'''
-__revision__ = '$Revision: 2480 $'
+__revision__ = '$Revision: 2546 $'
 
 import logging, time
 import Bcfg2.Client.Tools
@@ -83,15 +83,30 @@ class Frame:
         '''Set self.whitelist based on user interaction'''
         prompt = "Would you like to install %s: %s? (y/N): "
         rprompt = "Would you like to remove %s: %s? (y/N): "
+        if self.setup['remove']:
+            if self.setup['remove'] == 'all':
+                self.removal = self.extra
+            elif self.setup['remove'] == 'services':
+                self.removal = [entry for entry in self.extra if entry.tag == 'Service']
+            elif self.setup['remove'] == 'packages':
+                self.removal = [entry for entry in self.extra if entry.tag == 'Package']
+
         if self.setup['dryrun']:
-            self.logger.info("In dryrun mode: suppressing entry installation for:")
-            self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
-                              in self.states if not self.states[entry]])
+            updated = [entry for entry in self.states if not self.states[entry]]
+            if updated:
+                self.logger.info("In dryrun mode: suppressing entry installation for:")
+                self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
+                                  in updated])
+            if self.removal:
+                self.logger.info("In dryrun mode: suppressing entry removal for:")
+                self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
+                                  in self.removal])
+            self.removal = []
             return
         elif self.setup['interactive']:
             self.whitelist = promptFilter(prompt, [entry for entry in self.states \
                                                    if not self.states[entry]])
-            self.removal = promptFilter(rprompt, self.extra)
+            self.removal = promptFilter(rprompt, self.removal)
         elif self.setup['bundle']:
             # only install entries in specified bundle
             mbs = [bund for bund in self.config.findall('./Bundle') \
@@ -104,13 +119,6 @@ class Frame:
         else:
             # all systems are go
             self.whitelist = [entry for entry in self.states if not self.states[entry]]
-            if self.setup['remove']:
-                if self.setup['remove'] == 'all':
-                    self.removal = self.extra
-                elif self.setup['remove'] == 'services':
-                    self.removal = [entry for entry in self.extra if entry.tag == 'Service']
-                elif self.setup['remove'] == 'packages':
-                    self.removal = [entry for entry in self.extra if entry.tag == 'Package']
 
     def DispatchInstallCalls(self, entries):
         '''Dispatch install calls to underlying tools'''
@@ -131,9 +139,10 @@ class Frame:
             mods = self.modified
             mbundles = [struct for struct in self.config if struct.tag == 'Bundle' and \
                         [mod for mod in mods if mod in struct]]
-            self.logger.info("The Following Bundles have been modifed:")
-            self.logger.info([mbun.get('name') for mbun in mbundles])
-            self.logger.info("")
+            if mbundles:
+                self.logger.info("The Following Bundles have been modifed:")
+                self.logger.info([mbun.get('name') for mbun in mbundles])
+                self.logger.info("")
             tbm = [(t, b) for t in self.tools for b in mbundles]
             for tool, bundle in tbm:
                 try:
@@ -153,7 +162,7 @@ class Frame:
     def Remove(self):
         '''Remove extra entries'''
         for tool in self.tools:
-            extras = [entry for entry in self.removal if tool.canInstall(entry)]
+            extras = [entry for entry in self.removal if tool.handlesEntry(entry)]
             if extras:
                 try:
                     tool.Remove(extras)
@@ -171,7 +180,13 @@ class Frame:
 
         if ((self.states.values().count(False) == 0) and not self.extra):
             self.logger.info('All entries correct.')
-    
+
+    def ReInventory(self):
+        '''Recheck everything'''
+        if not self.setup['dryrun'] and self.setup['kevlar']:
+            self.logger.info("Rechecking system inventory")
+            self.Inventory()
+
     def Execute(self):
         '''Run all methods'''
         self.Inventory()
@@ -181,6 +196,10 @@ class Frame:
         self.Install()
         self.times['install'] = time.time()
         self.Remove()
+        self.times['remove'] = time.time()
+        if self.modified:
+            self.ReInventory()
+            self.times['reinventory'] = time.time()
         self.times['finished'] = time.time()
         self.CondDisplayState('final')
 
