@@ -1,5 +1,5 @@
 '''This is the bcfg2 support for apt-get'''
-__revision__ = '$Revision: 2418 $'
+__revision__ = '$Revision: 2546 $'
 
 import apt_pkg
 import os, re
@@ -41,6 +41,7 @@ class APT(Bcfg2.Client.Tools.PkgTool):
         for pkg in cache.Packages:
             if pkg.CurrentVer:
                 self.installed[pkg.Name] = pkg.CurrentVer.VerStr
+        self.extra = self.FindExtraPackages()
 
     def VerifyPackage(self, entry, modlist):
         '''Verify package for entry'''
@@ -51,8 +52,26 @@ class APT(Bcfg2.Client.Tools.PkgTool):
         if self.installed.has_key(entry.attrib['name']):
             if self.installed[entry.attrib['name']] == entry.attrib['version']:
                 if not self.setup['quick'] and entry.get('verify', 'true') == 'true':
-                    output = self.cmd.run("/usr/bin/debsums -s %s" % entry.get('name'))[1]
-                    if [filename for filename in output if filename not in modlist]:
+                    output = self.cmd.run("/usr/bin/debsums -as %s" % entry.get('name'))[1]
+                    if len(output) == 1 and "no md5sums for" in output[0]:
+                        self.logger.info("Package %s has no md5sums. Cannot verify" % \
+                                         entry.get('name'))
+                        return False
+                    files = []
+                    for item in output:
+                        if "checksum mismatch" in item:
+                            files.append(item.split()[-1])
+                        elif "can't open" in item:
+                            files.append(item.split()[5])
+                        else:
+                            self.logger.error("Got Unsupported pattern %s from debsums" \
+                                              % item)
+                            files.append(item)
+                    bad = [filename for filename in files if filename not in modlist]
+                    if bad:
+                        self.logger.info("Package %s failed validation. Bad files are:" % \
+                                         entry.get('name'))
+                        self.logger.info(bad)
                         return False
                 return True
             else:
@@ -63,11 +82,12 @@ class APT(Bcfg2.Client.Tools.PkgTool):
 
     def RemovePackages(self, packages):
         '''Deal with extra configuration detected'''
+        pkgnames = " ".join([pkg.get('name') for pkg in packages])
         if len(packages) > 0:
             self.logger.info('Removing packages:')
-            self.logger.info(packages)
-            self.cmd.run("apt-get remove -y --force-yes %s" % \
-                         " ".join(packages))
+            self.logger.info(pkgnames)
+            if self.cmd.run("apt-get remove -y --force-yes %s" % pkgnames)[0] == 0:
+                self.modified += packages
             self.RefreshPackages()
             self.extra = self.FindExtraPackages()
               
