@@ -1,8 +1,8 @@
 '''This contains all Bcfg2 Tool modules'''
-__revision__ = '$Revision: 2435 $'
+__revision__ = '$Revision: 2581 $'
 
-__all__ = ["APT", "Blast", "Chkconfig", "DebInit", "Encap", "PostInstall",
-           "POSIX", "RPM", "SMF", "SYSV"]
+__all__ = ["APT", "Blast", "Chkconfig", "DebInit", "Encap", "Portage",
+           "PostInstall", "POSIX", "RPM", "RcUpdate", "SMF", "SYSV"]
 
 import os, popen2, stat, sys, Bcfg2.Client.XML
 
@@ -42,11 +42,15 @@ class executor:
         cmdstat = -1
         while cmdstat == -1:
             runpipe.fromchild.flush()
-            moreOutput = runpipe.fromchild.read()
+            moreOutput = runpipe.fromchild.readline()
             if len(moreOutput) > 0:                
-                self.logger.debug('< %s' % moreOutput)
+                self.logger.debug('< %s' % moreOutput[:-1])
             output += moreOutput
             cmdstat = runpipe.poll()
+        for line in runpipe.fromchild.readlines():
+            if len(line) > 0:
+                self.logger.debug('< %s' % line[:-1])
+            output += line
 
         return (cmdstat, [line for line in output.split('\n') if line])
 
