@@ -9,11 +9,12 @@ from django.http import HttpResponse, HttpResponseRedirect
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth import logout
 
-from Hostbase.hostbase.models import *
+from Bcfg2.Server.Hostbase.hostbase.models import *
 from datetime import date
 from django.db import connection
 from django.shortcuts import render_to_response
-from Hostbase import settings, regex
+from django import forms
+from Bcfg2.Server.Hostbase import settings, regex
 import re
     
 attribs = ['hostname', 'whatami', 'netgroup', 'security_class', 'support',
@@ -152,34 +153,26 @@ def fill(template, hostdata, dnsdata=False):
     return template
 
 def edit(request, host_id):
-    """Edit general host information
-    Data is validated before being committed to the database"""
-    # fix bug when ip address changes, update the dns info appropriately
+    """Edit general host information"""
+    manipulator = Host.ChangeManipulator(host_id)
     changename = False
-    if request.GET.has_key('sub'):
+    if request.method == 'POST':
         host = Host.objects.get(id=host_id)
         if request.POST['hostname'] != host.hostname:
             oldhostname = host.hostname.split(".")[0]
-            host.hostname = request.POST['hostname']
-            host.save()
             changename = True
         interfaces = host.interface_set.all()
-        if not validate(request, False, host_id):
-            if (request.POST.has_key('outbound_smtp')
-                and not host.outbound_smtp or
-                not request.POST.has_key('outbound_smtp')
-                and host.outbound_smtp):
-                host.outbound_smtp = not host.outbound_smtp
-            # add validation for attribs here
-            # likely use a helper fucntion
-            for attrib in attribs:
-                if request.POST.has_key(attrib):
-                    host.__dict__[attrib] = request.POST[attrib].lower()
-            if request.POST.has_key('comments'):
-                host.comments = request.POST['comments']
-            if len(request.POST['expiration_date'].split("-")) == 3:
-                (year, month, day) = request.POST['expiration_date'].split("-")
-                host.expiration_date = date(int(year), int(month), int(day))
+        new_data = request.POST.copy()
+
+        errors = manipulator.get_validation_errors(new_data)
+        if not errors:
+
+            # somehow keep track of multiple interface change manipulators
+            # as well as multiple ip chnage manipulators??? (add manipulators???)
+            # change to many-to-many??????
+
+            # dynamically look up mx records?
+
             for inter in interfaces:
                 changetype = False
                 ips = IP.objects.filter(interface=inter.id)
@@ -217,8 +210,7 @@ def edit(request, host_id):
                     mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
                     if created:
                         mx.save()
-                    new_ip = IP(interface=inter, num=len(ips),
-                                ip_addr=request.POST['%dip_addr' % inter.id])
+                    new_ip = IP(interface=inter, ip_addr=request.POST['%dip_addr' % inter.id])
                     new_ip.save()
                     new_name = "-".join([host.hostname.split(".")[0],
                                          new_ip.ip_addr.split(".")[2]])
@@ -249,8 +241,7 @@ def edit(request, host_id):
                 mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
                 if created:
                     mx.save()
-                new_ip = IP(interface=new_inter, num=0,
-                            ip_addr=request.POST['ip_addr_new'])
+                new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new'])
                 new_ip.save()
                 new_name = "-".join([host.hostname.split(".")[0],
                                      new_ip.ip_addr.split(".")[2]])
@@ -278,8 +269,7 @@ def edit(request, host_id):
                                       hdwr_type=request.POST['hdwr_type_new'],
                                       dhcp=False)
                 new_inter.save()
-                new_ip = IP(interface=new_inter, num=0,
-                            ip_addr=request.POST['ip_addr_new'])
+                new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new'])
                 new_ip.save()
                 new_name = "-".join([host.hostname.split(".")[0],
                                      new_ip.ip_addr.split(".")[2]])
@@ -300,7 +290,7 @@ def edit(request, host_id):
             return HttpResponseRedirect('/hostbase/%s/' % host.id)
         else:
             return render_to_response('errors.html',
-                                      {'failures': validate(request, False, host_id),
+                                      {'failures': errors,
                                        'logged_in': request.session.get('_auth_user_id', False)})
     else:
         host = Host.objects.get(id=host_id)
@@ -518,8 +508,7 @@ def new(request):
                                   dhcp=request.POST.has_key('dhcp_new'))
             new_inter.save()
         if request.POST['mac_addr_new'] and request.POST['ip_addr_new']:
-            new_ip = IP(interface=new_inter,
-                        num=0, ip_addr=request.POST['ip_addr_new'])
+            new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new'])
             new_ip.save()
             mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
             if created:
@@ -547,8 +536,7 @@ def new(request):
                                   hdwr_type=request.POST['hdwr_type_new1'],
                                   dhcp=False)
             new_inter.save()
-            new_ip = IP(interface=new_inter, num=0,
-                        ip_addr=request.POST['ip_addr_new1'])
+            new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new1'])
             new_ip.save()
             mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
             if created:
@@ -578,8 +566,7 @@ def new(request):
                                   dhcp=request.POST.has_key('dhcp_new2'))
             new_inter.save()
         if request.POST['mac_addr_new2'] and request.POST['ip_addr_new2']:
-            new_ip = IP(interface=new_inter, num=0,
-                        ip_addr=request.POST['ip_addr_new2'])
+            new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new2'])
             new_ip.save()
             mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
             if created:
@@ -608,8 +595,7 @@ def new(request):
                                   hdwr_type=request.POST['hdwr_type_new2'],
                                   dhcp=False)
             new_inter.save()
-            new_ip = IP(interface=new_inter, num=0,
-                        ip_addr=request.POST['ip_addr_new2'])
+            new_ip = IP(interface=new_inter, ip_addr=request.POST['ip_addr_new2'])
             new_ip.save()
             mx, created = MX.objects.get_or_create(priority=settings.PRIORITY, mx=settings.DEFAULT_MX)
             if created:
@@ -682,19 +668,19 @@ def validate(request, new=False, host_id=None):
             and regex.host.match(request.POST['hostname'])):
         failures.append('hostname')
 
-    if not regex.printq.match(request.POST['printq']) and request.POST['printq']:
-        failures.append('printq')
+##     if not regex.printq.match(request.POST['printq']) and request.POST['printq']:
+##         failures.append('printq')
 
-    if not regex.user.match(request.POST['primary_user']):
-        failures.append('primary_user')
+##     if not regex.user.match(request.POST['primary_user']):
+##         failures.append('primary_user')
 
-    if (not regex.user.match(request.POST['administrator'])
-        and request.POST['administrator']):
-        failures.append('administrator')
+##     if (not regex.user.match(request.POST['administrator'])
+##         and request.POST['administrator']):
+##         failures.append('administrator')
 
-    if not (request.POST['location']
-            and regex.location.match(request.POST['location'])):
-        failures.append('location')
+##     if not (request.POST['location']
+##             and regex.location.match(request.POST['location'])):
+##         failures.append('location')
 
     if new:
         if (not regex.macaddr.match(request.POST['mac_addr_new'])
@@ -770,120 +756,126 @@ def zoneview(request, zone_id):
                                'logged_in': request.session.get('_auth_user_id', False)
                                })
 
-def zoneedit(request, zone_id):
-    if request.GET.has_key('sub'):
-        zone = Zone.objects.get(id=zone_id)
-        for attrib in zoneattribs:
-            if request.POST.has_key(attrib):
-                zone.__dict__[attrib] = request.POST[attrib]
-        count = 0
-        for nameserver in zone.nameservers.all():
-            ns, created = Nameserver.objects.get_or_create(name=request.POST['nameserver%i' % count])
-            if created or not (nameserver == ns):
-                ns.save()
-                zone.nameservers.add(ns)
-                zone.nameservers.remove(nameserver)
-            count += 1
-        count = 0
-        for mx in zone.mxs.all():
-            mrecord, created = MX.objects.get_or_create(priority=request.POST['priority%i' % count],
-                                                        mx=request.POST['mx%i' % count])
-            if created or not (mx == mrecord):
-                mrecord.save()
-                zone.mxs.add(mrecord)
-                zone.mxs.remove(mx)
-            count += 1
-        count = 0
-        for address in zone.addresses.all():
-            arecord, created = ZoneAddress.objects.get_or_create(ip_addr=request.POST['address%i' % count])
-            if created or not (arecord == address):
-                arecord.save()
-                zone.addresses.add(arecord)
-                zone.addresses.remove(address)
-            count += 1
-        zone.save()
-        if request.POST['new_nameserver']:
-            nameserver, created = Nameserver.objects.get_or_create(name=request.POST['new_nameserver'])
-            if created:
-                nameserver.save()
-            zone.nameservers.add(nameserver)
-        if request.POST['new_mx'] and request.POST['new_priority']:
-            mx, created = MX.objects.get_or_create(priority=request.POST['new_priority'],
-                                                   mx=request.POST['new_mx'])
-            if created:
-                mx.save()
-            zone.mxs.add(mx)
-        if request.POST['new_address'] and not request.POST['new_address'] == 'none':
-            address, created = ZoneAddress.objects.get_or_create(ip_addr=request.POST['new_address'])
-            if created:
-                address.save()
-            zone.addresses.add(address)
-        return HttpResponseRedirect('/hostbase/zones/%s/edit' % zone.id)
-    else:
-        zone = Zone.objects.get(id=zone_id)
-        return render_to_response('zoneedit.html',
-                                  {'zone': zone,
-                                   'nameservers': zone.nameservers.all(),
-                                   'mxs': zone.mxs.all(),
-                                   'addresses': zone.addresses.all(),
-                                   'logged_in': request.session.get('_auth_user_id', False)
-                                   })
-
 def zonenew(request):
-    if request.GET.has_key('sub'):
-        try:
-            Zone.objects.get(zone=request.POST['zone'])
-            return render_to_response('errors.html',
-                                      {'failures': ['%s already exists in database' % request.POST['zone']],
-                                       'logged_in': request.session.get('_auth_user_id', False)})
-
-        except:
-            zone = Zone(zone=request.POST['zone'])
-        for attrib in zoneattribs:
-            if request.POST.has_key(attrib):
-                zone.__dict__[attrib] = request.POST[attrib]
-        zone.serial = 1
-        zone.save()
-        for num in range(0,4):
-            if request.POST['nameserver%i' % num]:
-                ns, created = Nameserver.objects.get_or_create(name=request.POST['nameserver%i' % num])
-                if created:
-                    ns.save()
-                zone.nameservers.add(ns)
-        for num in range(0,2):
-            if request.POST['priority%i' % num] and request.POST['mx%i' % num]:
-                mrecord, created = MX.objects.get_or_create(priority=request.POST['priority%i' % num],
-                                                            mx=request.POST['mx%i' % num])
-                if created:
-                    mrecord.save()
-                zone.mxs.add(mrecord)
-        for num in range(0,2):
-            if request.POST['address%i' % num]:
-                arecord, created = ZoneAddress.objects.get_or_create(ip_addr=request.POST['address%i' % num])
-                if created:
-                    arecord.save()
-                zone.addresses.add(arecord)
-        return HttpResponseRedirect('/hostbase/zones/%s/' % zone.id)
+    manipulator = Zone.AddManipulator()
+    nsmanipulator = Nameserver.AddManipulator()
+    mxmanipulator = MX.AddManipulator()
+    addressmanipulator = ZoneAddress.AddManipulator()
+    
+    if request.method == 'POST':
+        new_data = request.POST.copy()
+        new_data['serial'] = '1'
+        errors = manipulator.get_validation_errors(new_data)
+        errors.update(check_zone_errors(request.POST.copy()))
+        if errors:
+            return render_to_response('errors.html', {'failures': errors})
+        else:
+            do_zone_add(manipulator, new_data)
+            return HttpResponseRedirect('/hostbase/zones/%s' % new_zone.id)
     else:
-        return render_to_response('zonenew.html',
-                                  {'nameservers': range(0,4),
-                                  'mxs': range(0,2),
-                                  'addresses': range(0,2),
-                                   'logged_in': request.session.get('_auth_user_id', False)
-                                   })
+        errors = new_data = {}
+
+    form = forms.FormWrapper(manipulator, {}, {})
+    nsform = forms.FormWrapper(nsmanipulator, {}, {})
+    mxform = forms.FormWrapper(mxmanipulator, {}, {})
+    aform = forms.FormWrapper(addressmanipulator, {}, {})
+    context = {'form': form,
+               'nsform': nsform,
+               'mxform': mxform,
+               'aform': aform,
+               }
+    return render_to_response('zonenew.html', context)
+
+def zoneedit(request, zone_id):
+    manipulator = Zone.ChangeManipulator(zone_id)
+    nsaddmanipulator = Nameserver.AddManipulator()
+    mxaddmanipulator = MX.AddManipulator()
+    addressaddmanipulator = ZoneAddress.AddManipulator()
+    zone = manipulator.original_object
+    nsmanipulators = [Nameserver.ChangeManipulator(ns.id)
+                      for ns in zone.nameservers.all()]
+    mxmanipulators = [MX.ChangeManipulator(mx.id) for mx in zone.mxs.all()]
+    addressmanipulators = [ZoneAddress.ChangeManipulator(address.id)
+                           for address in zone.addresses.all()]
+    if request.method == 'POST':
+        new_data = request.POST.copy()
+        new_data['serial'] = str(zone.serial)
+        errors = manipulator.get_validation_errors(new_data)
+        errors.update(check_zone_errors(request.POST.copy()))
+        if not errors:
+            do_zone_add(manipulator, new_data)
+            return HttpResponseRedirect('/hostbase/zones/%s' % zone.id)
+        else:
+            return render_to_response('errors.html', {'failures': errors})           
+    else:
+        errors = {}
+        new_data = manipulator.flatten_data()
+        
+    form = forms.FormWrapper(manipulator, new_data, errors)
+    nsforms = [forms.FormWrapper(nsm, nsm.flatten_data(), {}) for nsm in nsmanipulators]
+    mxforms = [forms.FormWrapper(mxm, mxm.flatten_data(), {}) for mxm in mxmanipulators]
+    aforms = [forms.FormWrapper(am, am.flatten_data(), {}) for am in addressmanipulators]
+    context = {'form': form,
+               'nsforms': nsforms,
+               'mxforms': mxforms,
+               'aforms': aforms,
+               'nsadd': forms.FormWrapper(nsaddmanipulator, {}, {}),
+               'mxadd': forms.FormWrapper(mxaddmanipulator, {}, {}),
+               'addadd': forms.FormWrapper(addressaddmanipulator, {}, {}),
+               'zone_id': zone_id,
+               'zone': zone.zone
+               }
+    return render_to_response('zoneedit.html', context)
+
+def do_zone_add(manipulator, new_data):
+    manipulator.do_html2python(new_data)
+    zone = manipulator.save(new_data)
+    for name in new_data.getlist('name'):
+        if name:
+            ns, created = Nameserver.objects.get_or_create(name=name)
+            zone.nameservers.add(ns)
+    priorities = new_data.getlist('priority')
+    for mx in new_data.getlist('mx'):
+        if priorities[0] and mx:
+            mxrecord, created = MX.objects.get_or_create(priority=priorities.pop(0), mx=mx)
+            zone.mxs.add(mxrecord)
+    for address in new_data.getlist('ip_addr'):
+        if address:
+            arecord, created = ZoneAddress.objects.get_or_create(ip_addr=address)
+            zone.addresses.add(arecord)
+
+def check_zone_errors(new_data):
+    errors = {}
+    for ns in new_data.getlist('name'):
+        errors.update(Nameserver.AddManipulator().get_validation_errors({'name':ns}))
+    for addr in new_data.getlist('ip_addr'): 
+        errors.update(ZoneAddress.AddManipulator().get_validation_errors({'ip_addr':addr}))
+    priorities = new_data.getlist('priority')
+    count = 0
+    for mx in new_data.getlist('mx'):
+        errors.update(MX.AddManipulator().get_validation_errors({'mx':mx, 'priority':priorities[count]})) 
+        count += 1
+    return errors
 
 ## login required stuff
 ## uncomment the views below that you would like to restrict access to
 
-## search = login_required(search)
-## look = login_required(look)
-## dns = login_required(dns)
+## uncomment the lines below this point to restrict access to pages that modify the database
+## anonymous users can still view data in Hostbase
+
 ## edit = login_required(edit)
 ## confirm = login_required(confirm)
 ## dnsedit = login_required(dnsedit)
 ## new = login_required(new)
 ## remove = login_required(remove)
-## zones = login_required(zones)
-## zoneview = login_required(zoneview)
 ## zoneedit = login_required(zoneedit)
 ## zonenew = login_required(zonenew)
+
+## uncomment the lines below this point to restrict access to all of hostbase
+
+## search = login_required(search)
+## look = login_required(look)
+## dns = login_required(dns)
+## zones = login_required(zones)
+## zoneview = login_required(zoneview)
+
