@@ -1,8 +1,8 @@
 '''This file provides the Hostbase plugin. It manages dns/dhcp/nis host information'''
-__revision__ = '$Revision: 2481 $'
+__revision__ = '$Revision: 2574 $'
 
 import sys, os
-os.environ['DJANGO_SETTINGS_MODULE'] = 'Hostbase.settings'
+os.environ['DJANGO_SETTINGS_MODULE'] = 'Bcfg2.Server.Hostbase.settings'
 from lxml.etree import Element, SubElement
 from syslog import syslog, LOG_INFO
 from Bcfg2.Server.Plugin import Plugin, PluginExecutionError, PluginInitError, DirectoryBacked
@@ -39,7 +39,7 @@ import re
 class Hostbase(Plugin):
     '''The Hostbase plugin handles host/network info'''
     __name__ = 'Hostbase'
-    __version__ = '$Id: Hostbase.py 2481 2006-11-10 18:41:52Z raffenet $'
+    __version__ = '$Id: Hostbase.py 2574 2006-12-08 21:34:03Z raffenet $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     filepath = '/etc/bind'
 
@@ -68,11 +68,8 @@ class Hostbase(Plugin):
                           }
         self.Entries['ConfigFile'] = {}
         self.__rmi__ = ['rebuildState']
-        try:
-            self.rebuildState(None)
-        except:
-            raise PluginInitError
-                
+        self.rebuildState(None)
+                 
     def FetchFile(self, entry, metadata):
         '''Return prebuilt file data'''
         fname = entry.get('name').split('/')[-1]
@@ -238,9 +235,10 @@ class Hostbase(Plugin):
 
         reversenames.sort()
         for filename in reversenames:
+            print filename
             originlist = []
             cursor.execute("""
-            SELECT h.hostname, p.ip_addr, p.num FROM ((hostbase_host h
+            SELECT h.hostname, p.ip_addr FROM ((hostbase_host h
             INNER JOIN hostbase_interface i ON h.id = i.host_id)
             INNER JOIN hostbase_ip p ON i.id = p.interface_id)
             WHERE p.ip_addr LIKE '%s%%%%' AND h.status = 'active' ORDER BY p.ip_addr
@@ -253,7 +251,7 @@ class Hostbase(Plugin):
                  if (".".join([ip[1].split(".")[2], filename[0]]),
                      ".".join([filename[1], ip[1].split(".")[2]])) not in originlist]
                 for origin in originlist:
-                    hosts = [host.__add__((host[1].split("."), host[0].split(".", 1)))
+                    hosts = [(host[1].split("."), host[0].split(".", 1))
                              for host in reversehosts
                              if host[1].rstrip('0123456789').rstrip('.') == origin[1]]
                     context = Context({
@@ -264,7 +262,7 @@ class Hostbase(Plugin):
                     self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
             else:
                 originlist = [filename[0]]
-                hosts = [host.__add__((host[1].split("."), host[0].split(".", 1)))
+                hosts = [(host[1].split("."), host[0].split(".", 1))
                          for host in reversehosts]
                 context = Context({
                     'hosts': hosts,
@@ -272,6 +270,7 @@ class Hostbase(Plugin):
                     'fileorigin': None,
                     })        
                 self.filedata['%s.rev' % filename[0]] += self.templates['reverseapp'].render(context)
+            print hosts
             self.Entries['ConfigFile']['%s/%s.rev' % (self.filepath, filename[0])] = self.FetchFile
 
     def buildDHCP(self):
